"use strict";
/* eslint-disable no-console */
Object.defineProperty(exports, "__esModule", { value: true });
const plugin_ui_utils_1 = require("@homebridge/plugin-ui-utils");
const winix_api_1 = require("winix-api");
class PluginUiServer extends plugin_ui_utils_1.HomebridgePluginUiServer {
    constructor() {
        super();
        this.onRequest('/login', this.login.bind(this));
        this.onRequest('/discover', this.discoverDevices.bind(this));
        this.ready();
    }
    async login({ email, password }) {
        console.log(`Logging in with email '${email}' and password`);
        let auth;
        try {
            auth = await winix_api_1.WinixAuth.login(email, password, 3);
        }
        catch (e) {
            const message = getErrorMessage(e);
            console.error(message);
            throw new plugin_ui_utils_1.RequestError(message, { status: 400 });
        }
        return {
            username: email,
            refreshToken: auth.refreshToken,
            userId: auth.userId,
        };
    }
    async discoverDevices(auth) {
        let winixDevices;
        try {
            const account = await winix_api_1.WinixAccount.fromExistingAuth(auth);
            winixDevices = await account.getDevices();
        }
        catch (e) {
            const message = getErrorMessage(e);
            console.error(message);
            throw new plugin_ui_utils_1.RequestError(message, { status: 500 });
        }
        const devices = winixDevices.map(({ deviceAlias, deviceId, modelName }) => {
            return { deviceId, deviceAlias, modelName };
        });
        return { devices };
    }
}
function getErrorMessage(error) {
    return error instanceof Error ? error.message : String(error);
}
function startPluginUiServer() {
    return new PluginUiServer();
}
startPluginUiServer();
